from html.parser import HTMLParser
from textwrap import fill
from typing import Final

BLOCK_ELEMENTS: Final = frozenset(("h1", "h2", "h3", "h4", "h5", "h6", "p", "br"))


class _HTMLFilter(HTMLParser):
    def __init__(self):
        self.paragraphs = [""]
        super().__init__()

    def handle_starttag(self, tag, attrs):
        if tag in BLOCK_ELEMENTS and self.paragraphs[-1] != "":
            self.paragraphs.append("")

    def handle_data(self, data):
        self.paragraphs[-1] += data


def strip_html(source: str, wrap_width: int = 79):
    """
    Returns plain-text version of an HTML document.
    """

    html_filter = _HTMLFilter()
    html_filter.feed(source)
    return "\n\n".join(fill(para, wrap_width).strip() for para in html_filter.paragraphs)
